/*
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
 */
accountantApp

.factory("Entries", [ "$resource", function($resource)  {
    return $resource(
        "/api/entries/:id", {
            id: "@id"
        }
    );
}])

.controller(
  "CategoryChartController", [
    "$rootScope", "$scope", "$http", "$routeParams",
    function($rootScope, $scope, $http, $routeParams) {

  var colors = Highcharts.getOptions().colors;
  $scope.revenueColor = colors[2];
  $scope.expenseColor = colors[3];

  // Configure pie chart for categories.
  $scope.config = {
    options: {
      chart: {
        type: 'pie',
        animation: {
          duration: 500
        }
      },
      plotOptions: {
        pie: {
          startAngle: -90
        },
        series: {
          allowPointSelect: 0
        }
      },
      tooltip: {
        valueDecimals: 2,
        valueSuffix: '€'
      }
    },
    yAxis: {
      title: {
        text: "Categories"
      }
    },
    title: {
      text: "Répartition dépenses/recettes"
    },
    series: [{
      name: "Value",
      data: [],
      innerSize: '33%',
      size: '60%',
      dataLabels: {
        formatter: function() {
          return this.point.name;
        },
        distance: -40
      }
    }, {
      name: "Value",
      data: [],
      innerSize: '66%',
      size: '60%',
      dataLabels: {
        formatter: function() {
          return this.point.name !== null && this.percentage >= 2.5 ? this.point.name : null;
        },
      }
    }]
  };

  $scope.brightenColor = function(color) {
    brightness = 0.2;

    return Highcharts.Color(color).brighten(brightness).get()
  };

  // Load categories, mainly to populate the pie chart.
  $scope.load = function(begin, end) {
    $scope.config.loading = true;

    $http.get("/api/categories", {
      params: {
        account: $routeParams.accountId,
        begin: begin.format('YYYY-MM-DD'),
        end: end.format('YYYY-MM-DD')
      }
    }).success(function(data) {
      var expenses = [], revenues = [];

      var expenseColor = $scope.brightenColor($scope.expenseColor);
      var revenueColor = $scope.brightenColor($scope.revenueColor);

      angular.forEach(angular.fromJson(data), function(category) {
        expenses.push({
          name: category.category,
          y: -category.expenses,
          color: expenseColor
        });

        revenues.push({
          name: category.category,
          y: category.revenues,
          color: revenueColor
        });
      });

      // Note: expenses and revenues must be in the same order than in series[0].
      $scope.config.series[1].data = revenues.concat(expenses);

      $scope.config.loading = false;
    });
  };

  /*
   * Get account status.
   */
  $scope.getAccountStatus = function(begin, end) {
    $http.get("/api/accounts/" + $routeParams.accountId, {
      params: {
        begin: begin.format('YYYY-MM-DD'),
        end: end.format('YYYY-MM-DD')
      }
    }).success(function(account) {
      // Emit accountLoadedEvent.
      $scope.$emit("accountLoadedEvent", account);

      // Update pie chart subtitle with Balance.
      $scope.config.subtitle = {
        text: "Balance: " + account.balance
      };

      $scope.config.series[0].data = [{
        name: "Revenues",
        y: account.revenues,
        color: $scope.revenueColor
      }, {
        name: "Expenses",
        y: -account.expenses,
        color: $scope.expenseColor,
      }];
    });
  };

  // Reload categories and account status on range selection.
  $rootScope.$on("rangeSelectedEvent", function(e, args) {
    $scope.load(args.begin, args.end);
    $scope.getAccountStatus(args.begin, args.end);
  });
}])

.controller(
  "SoldChartController", [
    "$rootScope", "$scope", "$http", "$routeParams",
    function($rootScope, $scope, $http, $routeParams) {

  // Configure chart for entries.
  $scope.config = {
    options: {
      chart: {
        zoomType: 'x'
      },
      rangeSelector: {
        buttons: [{
          type: 'month',
          count: 1,
          text: "1m"
        }, {
          type: "month",
          count: 3,
          text: "3m"
        }, {
          type: "month",
          count: 6,
          text: "6m"
        }, {
          type: "year",
          count: 1,
          text: "1y"
        }, {
          type: "all",
          text: "All"
        }],
        selected: 0,
      },
      navigator: {
        enabled: true
      },
      tooltip: {
        crosshairs: true,
        shared: true,
        valueDecimals: 2,
        valueSuffix: '€'
      },
      scrollbar: {
        liveRedraw: false
      }
    },
    series: [{
      type: "ohlc",
      name: "Sold",
      data: [],
      dataGrouping : {
        units : [[
          'week', // unit name
          [1] // allowed multiples
        ], [
          'month',
          [1, 2, 3, 4, 6]
        ]]
      }
    }],
    title: {
      text: "Sold evolution"
    },
    xAxis: {
      type: "datetime",
      dateTimeLabelFormats: {
        month: "%e. %b",
        year: "%Y"
      },
      minRange: 3600 * 1000 * 24 * 14, // 2 weeks
      events: {
        afterSetExtremes: function(e) {
          $scope.selectRange(moment.utc(e.min), moment.utc(e.max));
        }
      },
      currentMin: moment.utc().startOf('month'),
      currentMax: moment.utc().endOf('month')
    },
    yAxis: {
      plotLines: [{
        color: "orange",
        width: 2,
        value: 0.0
      }, {
        color: "red",
        width: 2,
        value: 0.0
      }]
    },
    useHighStocks: true
  };

  $scope.selectRange = function(begin, end) {
    $scope.$emit("rangeSelectedEvent", {begin: begin, end: end});
  };

  $scope.loadSolds = function() {
    $scope.config.loading = true;

    $http.get("/api/solds", {
      params: {
        account: $routeParams.accountId,
      }
    }).success(function(data) {
      $scope.config.series[0].data = [];

      angular.forEach(data, function(entry) {
        $scope.config.series[0].data.push([
          moment.utc(entry.operation_date).valueOf(),
          entry.open, entry.high, entry.low, entry.close
        ]);
      });

      $scope.$emit("rangeSelectedEvent", {
        begin: $scope.config.xAxis.currentMin,
        end: $scope.config.xAxis.currentMax
      });

      $scope.config.loading = false;
    });
  };

  // Reload solds when an entry is saved.
  $rootScope.$on("entrySavedEvent", function(e, entry) {
    $scope.loadSolds();
  });

  // Update authorized overdraft on account loading.
  $rootScope.$on("accountLoadedEvent", function(e, account) {
    $scope.config.yAxis.plotLines[1].value = account.authorized_overdraft;
  });

  // Select beginning and end of month.
  $scope.loadSolds();
}])

.controller(
    "EntryController", [
        "$scope", "$http", "$rootScope", "$filter", "$routeParams", "notificationService", "Entries",
        function($scope, $http, $rootScope, $filter, $routeParams, notificationService, Entries) {
    // Entry store and selection
    $scope.entries = [];

    // Function to reset the new entry.
    $scope.addEntry = function() {
      if(!$scope.inserted) {
        $scope.inserted = new Entries();
        $scope.inserted.account_id = $routeParams.accountId;
        $scope.entries.splice(0, 0, $scope.inserted);
      }
    };

    // Function to load entries from server for a specific account and month.
    $scope.loadEntries = function(begin, end) {
        // Clean up selected entry.
        $scope.selectedItem = null;
        $scope.savedItem = null;

        $scope.entries = Entries.query({
            account: $routeParams.accountId,
            begin: begin.format('YYYY-MM-DD'),
            end: end.format('YYYY-MM-DD')
        }, function(data) {
            $scope.$emit("entriesLoadedEvent", {entries: data});
        });
    };

    // Cancel current editing entry or clears field if a new one.
    $scope.cancelEditEntry = function(entry, rowform) {
      if(entry == $scope.inserted) {
        $scope.entries.splice(0, 1);
      } else {
        rowform.$cancel();
      }
    };

    /*
     * Toggle pointed indicator for an entry.
     */
    $scope.togglePointedEntry = function(entry, rowform) {
        entry.pointed = !entry.pointed;

        // Save entry if not editing it.
        if(!rowform.$visible) {
          $scope.saveEntry(entry);
        }
    };

    /*
     * Toggle cancel indicator for an entry.
     */
    $scope.toggleCanceledEntry = function(entry) {
        entry.canceled = !entry.canceled;

        $scope.saveEntry(entry);
    };

    /*
     * Save an entry and emit entrySavedEvent.
     */
    $scope.saveEntry = function($data, $index) {
      // Check if $data is already a resource.
      var entry;

      if($data.$save) {
        entry = $data;
      } else {
        entry = $scope.entries[$index];
        entry = angular.merge(entry, $data);
      }

      entry.confirmed = true;

      var promise = entry.$save();

      if(entry == $scope.inserted) {
        promise = promise.then(function(data) {
          $scope.inserted = false;

          return data;
        });
      }

      return promise.then(function(data) {
        notificationService.success("Entry #" + data.id + " saved.");
        $scope.$emit("entrySavedEvent", data);

        return data;
      });
    };

    // Reload entries on range selection.
    $rootScope.$on("rangeSelectedEvent", function(e, args) {
      $scope.loadEntries(args.begin, args.end);
    });
}]);
